  ----------------------------------------------------------------------------------------------------------------------
    -- This program is free software: you can redistribute it and/or modify
    -- it under the terms of the GNU General Public License as published by
    -- the Free Software Foundation, either version 3 of the License, or
    -- (at your option) any later version.
	
    -- This program is distributed in the hope that it will be useful,
    -- but WITHOUT ANY WARRANTY; without even the implied warranty of
    -- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    -- GNU General Public License for more details.

    -- You should have received a copy of the GNU General Public License
    -- along with this program.  If not, see <http://www.gnu.org/licenses/>.
----------------------------------------------------------------------------------------------------------------------


-- [[ Tooltips.lua ]]
-- Dynamic tooltip text functions that can be hooked to the appropriate events by the GUI controller

local addonName, TotalAP = ...

if not TotalAP then return end

local L = LibStub("AceLocale-3.0"):GetLocale("TotalAP", false)

-- This is the mouseover-tooltip for progress bars, based on Blizzard's ForgeDisplay (icon to the top-left of the ArtifactFrame)
local function ArtifactKnowledgeTooltipFunction(self, button, hide)
--	TotalAP.ChatMsg(self:GetName())
	
	local specID = tonumber((self:GetName()):match("(%d+)$")) -- e.g., TotalAPProgressBar1 for spec = 1

	
	-- local artifactName = TotalAP.DBHandler.GetArtifactNameForSpec(specID) -- might require call to GetItemInfo(id)?
	
	-- TODO: DRY
		local characterName = UnitName("player")
		local realm = GetRealmName()
		local key = format("%s - %s", characterName, realm)	 

	local cache = TotalArtifactPowerCache
		
	-- Cached values are sufficient, as the tooltip will only be shown for specs that have them (since it is attached to the ProgressBars, not the background)
	
	-- Retrieve this spec's artifact name from the "ArtifactsDB"
	local _, _, classID = UnitClass("player"); -- 1 to 12
	local specArtifacts = TotalArtifactPowerDB["artifacts"][classID][specID] -- TODO
	
	local _, itemID = next(specArtifacts[1]) -- index 1 => Pick first (lowest item ID) match -> should be mainhand weapon (but probably doesn't matter, because MH + OH are created from just one item when equipped)
	local artifactName = GetItemInfo(itemID)  --  since the spec has been cached the item name should be available already... otherweise, the first line won't show until the tooltip is displayed again and the server sent the item's name
		--Note: This is NOT the individual weapon's name for MH + OH artifacts (e.g., "Warswords of the Valarjar" = correct, instead of Helya/Odyn's Fury/Wrath) // TODO: DBHandler
	
	-- Load cached values
	local numTraitsPurchased = cache[key][specID]["numTraitsPurchased"] -- TODO: ArtifactCacheHandler
	local inBagsTotalAP = TotalAP.sharedLocals["inBagsTotalAP"] -- TODO
	local maxAvailableAP = cache[key][specID]["thisLevelUnspentAP"] + inBagsTotalAP
	local tier = cache[key][specID]["artifactTier"]
	
	-- Calculate progress from cached values
	local maxAttainableRank = numTraitsPurchased + TotalAP.ArtifactInterface.GetNumRanksPurchasableWithAP(numTraitsPurchased, maxAvailableAP, tier) 
	local progressPercent = TotalAP.ArtifactInterface.GetProgressTowardsNextRank(numTraitsPurchased, maxAvailableAP, tier)
	local knowledgeLevel = TotalAP.ArtifactInterface.GetArtifactKnowledgeLevel() 
	
	-- Calculate shipment data (for Artifact Research Notes)
	local shipmentsReady = TotalAP.ArtifactInterface.GetNumAvailableResearchNotes()
	local shipmentsTotal = 2 -- Could use the interface, but no more than 2 can actually be queued anyway
	local timeleft, timeLeftString = TotalAP.ArtifactInterface.GetTimeUntilNextResearchNoteIsReady()

	  GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
      if artifactName then GameTooltip:AddLine(format("%s", artifactName), 230/255, 204/255, 128/255) end
      
	  -- if AK not cached then don't display tooltip? Or display AK only / no progress for weapon
      -- TODO: Locale table (format %s)
	  -- TODO: Split into two tooltips -> this => ProgressBarTooltip and AK Tooltip for separate research bar/indicator (not yet implemented)
      GameTooltip:AddLine("Total Ranks Purchased: " ..  numTraitsPurchased, 1, 1, 1)
      GameTooltip:AddLine(format("%.2f%% towards Rank %d",  progressPercent, maxAttainableRank)) 
      GameTooltip:AddLine("\nArtifact Knowledge Level: " .. knowledgeLevel, 1, 1, 1)
      GameTooltip:AddLine("Shipments ready for pickup: " .. shipmentsReady .. "/" .. shipmentsTotal)
      if timeLeft then GameTooltip:AddLine("Next pickup in: " .. timeLeftString) end
	
	if hide then
		GameTooltip:Hide()
	else
		GameTooltip:Show()
	end
	
end

local function ShowArtifactKnowledgeTooltip(self, button)

	ArtifactKnowledgeTooltipFunction(self, button)

end

local function HideArtifactKnowledgeTooltip(self, button)

	ArtifactKnowledgeTooltipFunction(self, button, true)

end



-- Displayed on mouseover for the spec icons (used to activate/ignore specs)
local function SpecIconTooltipFunction(self, button, hide)
	
	local specNo = (self:GetName()):match("(%d)$") -- e.g., TotalAPSpecIconButton1 for spec = 1
	
	 -- On mouseover, show message that spec can be changed by clicking (unless it's the currently active spec)
	
	-- Show tooltip "Click to change spec" or sth. TODO
	GameTooltip:SetOwner(self, "ANCHOR_CURSOR");
	
	local _, specName = GetSpecializationInfo(specNo);
	GameTooltip:SetText(format(L["Specialization: %s"], specName), nil, nil, nil, nil, true);
	
	if i == GetSpecialization() then 
		GameTooltip:AddLine(L["This spec is currently active"], 0/255, 255/255, 0/255);
	else
		GameTooltip:AddLine(L["Click to activate"],  0/255, 255/255, 0/255);
	end	
	
	-- TODO: Colours could be set via Config (once it is implemented) ?
	--GameTooltip:AddLine(L["Right-click to ignore this spec"],  204/255, 85/255, 0/255);
	--GameTooltip:AddLine(L["Right-click to ignore this spec"],  0/255, 114/255, 202/255);
	--GameTooltip:AddLine(L["Right-click to ignore this spec"],  202/255, 0/255, 5/255);
	GameTooltip:AddLine(L["Right-click to ignore this spec"],  255/255, 32/255, 32/255) -- This is RED_FONT_COLOR_CODE from FrameXML
		
	
	if hide then
		GameTooltip:Hide()
	else
		GameTooltip:Show()
	end
	
end


local function ShowSpecIconTooltip(self, button)

	SpecIconTooltipFunction(self, button)

end

local function HideSpecIconTooltip(self, button)

	SpecIconTooltipFunction(self, button, true)

end


TotalAP.GUI.Tooltips = {
	ShowSpecIconTooltip = ShowSpecIconTooltip,
	HideSpecIconTooltip = HideSpecIconTooltip,
	ShowArtifactKnowledgeTooltip = ShowArtifactKnowledgeTooltip,
	HideArtifactKnowledgeTooltip = HideArtifactKnowledgeTooltip,
}

return TotalAP.GUI.Tooltips